;+
; IGFLib; autor: Sylwester Arabas (sarabas@igf.fuw.edu.pl); licencja: GPL      ;   
;                                                                              ;
; Metoda-procedura interpolujca wartoci temperatury z depeszy METAR na       ;
; regularn siatk n x m. Metoda zapisuje wynik interpolacji do przekazanych   ;
; w argumencie zmiennych:                                                      ;
; - grid_lat - szerokoci geograficzne punktw siatki                          ;
; - grid_lon - dugoci geograficzne punktw siatki                            ;
; - grid_val - wartoci temperatury w punktach siatki                          ;
; Przy obecnej fladze /barnes, metoda interpoluje przy uyciu algorytmu        ;
; iteracyjnego Barnesa, domylnie liczona jest rednia waona odlegoci      ;
; od punktu pomiarowego dla punktw w promieniu 1000km
;-
pro IGFLibMetarMap::interpolateToGrid, n, m, grid_lat, grid_lon, grid_val, $
  barnes=barnes, b_niters = itrs, b_gamma = gmma, b_lzero = lzro

  grid_val = dblarr(n, m)
  grid_lat = self.latrange[0] + dindgen(n) * (self.latrange[1] - self.latrange[0]) / (n-1)
  grid_lon = self.lonrange[0] + dindgen(m) * (self.lonrange[1] - self.lonrange[0]) / (m-1)

  if keyword_set(barnes) then begin
    ; iteracyjna interpolacja algorytmem Barnesa 
    notnan = where(finite(*(self.stations_temp)), notnan_n)
    nghbrs = lonarr(notnan_n, 4) 
    nghbrs_norm = dblarr(notnan_n)
    dstnc = dblarr(notnan_n, n * m)
    est_val = dblarr(notnan_n)

    ; trzy "look-up tables": 
    ; - dstnc: odlegoci pomidzy stacjami i punktami siatki [w km!]
    ; - nghbrs: punktw siatki- ssiadw stacji
    ; - nghbrs_norm: sumy odlegoci pomidzy punktami siatki a czterema ssiadami
    for ilat = 0l, n - 1l do for ilon = 0l, m - 1l do for k = 0l, notnan_n - 1l do begin
      i = ilon * n + ilat
      dstnc[k, i] = .001 * map_2points(grid_lon[ilon], grid_lat[ilat], $
        (*(self.stations_lons))[notnan[k]], (*(self.stations_lats))[notnan[k]], /meters)
      x = grid_lon[ilon] lt (*(self.stations_lons))[notnan[k]]
      y = grid_lat[ilat] lt (*(self.stations_lats))[notnan[k]] 
      nghbrs[k, x * 2 + y] = dstnc[k, nghbrs[k, x * 2 + y]] gt dstnc[k, i] ? i : nghbrs[k, x * 2 + y]
    endfor ; ilat, ilon, k
    for k = 0l, notnan_n - 1l do nghbrs_norm[k] = total(1./dstnc[k, nghbrs[k, *]])

    for j = 0l, itrs - 1l do begin
      l2 = gmma^(j + 1) * lzro^2
      if j ne 0l then for k = 0l, notnan_n - 1l do $
        est_val[k] = total(grid_val[nghbrs[k, *]] / dstnc[k, nghbrs[k, *]]) / nghbrs_norm[k]
      for i = 0l, n * m - 1l do begin
        w_total = 0d 
        corr = 0d
        for k = 0l, notnan_n - 1l do begin
          w = exp(-.5 * dstnc[k, i]^2 / l2)
          w_total += w
          corr += ((*(self.stations_temp))[notnan[k]] - est_val[k]) * w
        endfor ; k 
        corr /= w_total
        if finite(corr) then grid_val[i] += corr
      endfor ; i
    endfor ; j

  endif else begin
    ; rednie waone odwrotnoci odlegoci od stacji pomiarowej
    for ni = 0L, n - 1L do begin
      for mi = 0L, m - 1L do begin
        weight_total = 0d
        for si = 0L, self.stations_n - 1L do begin
          ; pominicie stacji z wartociami NaN
          if finite((*(self.stations_temp))[si]) then begin
            weight = 1d / map_2points(grid_lon[mi], grid_lat[ni], $
              (*(self.stations_lons))[si], (*(self.stations_lats))[si], /meters)
            ; pominicie stacji w odlegoci wikszej od 1000 km
            if weight gt .000001d then begin
              grid_val[ni, mi] += (*(self.stations_temp))[si] * weight
              weight_total += weight
            endif ; dist
          endif ; finite
        endfor ; si
        grid_val[ni, mi] /= weight_total
      endfor ; mi
    endfor ; ni
  endelse ; barnes

end
